import numpy as np
from cffi import FFI
import timeit
import random

def time_it_start():
    return timeit.default_timer()

def time_it_stop(before):
    duration = timeit.default_timer() - before
    print(f"Time: {duration:.3f} s")

def random_array(size=10):
    a = np.arange(size, dtype=np.int32)
    for i in range(size):
        a[i] = random.randint(1,size)
    return a

def python_sort(l):
    print("=== python sort ===")
    t = time_it_start()
    l.sort()
    time_it_stop(t)
    return l

def numpy_sort(arr):
    print("=== numpy sort ===")
    t = time_it_start()
    sorted = np.sort(arr)
    time_it_stop(t)
    return sorted

def rust_sort(arr, C):
    print("=== rust sort ===")
    t = time_it_start()
    C.sort(arr.__array_interface__['data'][0], np.size(arr))
    time_it_stop(t)
    return arr

def rust_sort_par(arr, C):
    print("=== rust sort parallel ===")
    t = time_it_start()
    C.sort_par(arr.__array_interface__['data'][0], np.size(arr))
    time_it_stop(t)
    return arr

ffi = FFI()
ffi.cdef("""
        void sort(uint64_t,uint64_t);
        void sort_par(uint64_t,uint64_t);
""")

C = ffi.dlopen("rust_functions/target/release/librust_functions.so")

array_length = 50 * 1000 * 1000;
arr = random_array(array_length)

out0 = python_sort(arr.tolist())
out1 = numpy_sort(arr.copy())
out2 = rust_sort(arr.copy(), C)
out3 = rust_sort_par(arr.copy(), C)

print(out0[:10],out0[-10:])
print(out1)
print(out2)
print(out3)
